@@ -1,6 +1,8 @@
-##### KNOWN BUGS #####
-# - It does not currently work to pass groupColumn one of the variables that is being plotted
-#   as an axis
+if(getRversion() >= "2.15.1") {
+  utils::globalVariables(c("variable", "value", "ggally_splineFactor"))
+}
+
+
 
 #' ggparcoord - A ggplot2 Parallel Coordinate Plot
 #'
@@ -67,6 +69,7 @@
 #' @param order method used to order the axes (see Details)
 #' @param showPoints logical operator indicating whether points should be
 #'   plotted or not
+#' @param splineFactor logical or numeric operator indicating whether spline interpolation should be used.  Numeric values will multiplied by the number of columns, \code{TRUE} will default to cubic interpolation, \code{\link[base]{AsIs}} to set the knot count directly and \code{0}, \code{FALSE}, or non-numeric values will not use spline interpolation.
 #' @param alphaLines value of alpha scaler for the lines of the parcoord plot or a column name of the data
 #' @param boxplot logical operator indicating whether or not boxplots should
 #'   underlay the distribution of each variable
@@ -77,6 +80,8 @@
 #' @author Jason Crowley \email{crowley.jason.s@@gmail.com}, Barret Schloerke \email{schloerke@@gmail.com}, Di Cook \email{dicook@@iastate.edu}, Heike Hofmann \email{hofmann@@iastate.edu}, Hadley Wickham \email{h.wickham@@gmail.com}
 #' @return ggplot object that if called, will print
 #' @import plyr
+#' @importFrom reshape melt melt.data.frame
+#' @importFrom stats complete.cases sd median mad lm spline
 #' @export
 #' @examples
 #' # use sample of the diamonds data for illustrative purposes
@@ -97,12 +102,13 @@
 #'
 #' # utilize ggplot2 aes to switch to thicker lines
 #' gpd <- ggparcoord(data = diamonds.samp,columns = c(1,5:10),groupColumn = 2,
-#'   title="Parallel Coord. Plot of Diamonds Data",mapping = aes(size = 1))
+#'   title="Parallel Coord. Plot of Diamonds Data",mapping = ggplot2::aes(size = 1))
 #' # gpd
 #'
 #' # basic parallel coord plot of the msleep data, using 'random' imputation and
 #' # coloring by diet (can also use variable names in the columns and groupColumn
 #' # arguments)
+#' data(msleep, package="ggplot2")
 #' gpd <- ggparcoord(data = msleep, columns = 6:11, groupColumn = "vore", missing =
 #'   "random", scale = "uniminmax")
 #' # gpd
@@ -132,9 +138,19 @@
 #'   showPoints = TRUE, title = "Parallel Coordinate Plot for the Iris Data",
 #'   alphaLines = "alphaLevel")
 #' # gpd
+#'
+#' ## Use splines on values, rather than lines (all produce the same result)
+#' columns <- c(1, 5:10)
+#' gpd <- ggparcoord(diamonds.samp, columns, groupColumn = 2, splineFactor = TRUE)
+#' # gpd
+#' gpd <- ggparcoord(diamonds.samp, columns, groupColumn = 2, splineFactor = 3)
+#' # gpd
+#' splineFactor <- length(columns) * 3
+#' gpd <- ggparcoord(diamonds.samp, columns, groupColumn = 2, splineFactor = I(splineFactor))
+#' # gpd
 ggparcoord <- function(
   data,
-  columns,
+  columns      = 1:ncol(data),
   groupColumn  = NULL,
   scale        = "std",
   scaleSummary = "mean",
@@ -142,6 +158,7 @@ ggparcoord <- function(
   missing      = "exclude",
   order        = columns,
   showPoints   = FALSE,
+  splineFactor = FALSE,
   alphaLines   = 1,
   boxplot      = FALSE,
   shadeBox     = NULL,
@@ -153,82 +170,115 @@ ggparcoord <- function(
   ### Error Checking ###
   if(is.null(groupColumn)) {
     if(any(tolower(order) %in% c("anyclass","allclass"))) {
-      stop("can't use the order methods anyClass or allClass without specifying groupColumn")
+      stop("can't use the 'order' methods anyClass or allClass without specifying groupColumn")
     }
-  } else if(!((length(groupColumn) == 1) && (is.numeric(groupColumn) || is.character(groupColumn)))) {
-    stop("invalid value for groupColumn; must be a single numeric or character index")
+  } else if(
+    !((length(groupColumn) == 1) && (is.numeric(groupColumn) || is.character(groupColumn)))
+  ) {
+    stop("invalid value for 'groupColumn'; must be a single numeric or character index")
   }
 
   if(!(tolower(scale) %in% c("std","robust","uniminmax","globalminmax","center","centerobs"))) {
-    stop("invalid value for scale; must be one of 'std','robust','uniminmax','globalminmax','center', or 'centerObs'")
+    stop(str_c(
+      "invalid value for 'scale'; must be one of ",
+      "'std','robust','uniminmax','globalminmax','center', or 'centerObs'"
+    ))
   }
 
   if(!(centerObsID %in% 1:dim(data)[1])) {
-    stop("invalid value for centerObsID; must be a single numeric row index")
+    stop("invalid value for 'centerObsID'; must be a single numeric row index")
   }
 
   if(!(tolower(missing) %in% c("exclude","mean","median","min10","random"))) {
-    stop("invalid value for missing; must be one of 'exclude','mean','median','min10','random'")
+    stop("invalid value for 'missing'; must be one of 'exclude','mean','median','min10','random'")
   }
 
-  if(!(is.numeric(order) || (is.character(order) && (order %in% c('skewness','allClass','anyClass',
-    'Outlying','Skewed','Clumpy', 'Sparse', 'Striated', 'Convex', 'Skinny', 'Stringy','Monotonic'))))) {
-    stop("invalid value for order; must either be a vector of column indices or one of 'skewness','allClass','anyClass','Outlying','Skewed','Clumpy','Sparse','Striated','Convex','Skinny','Stringy','Monotonic'")
+  if(!(
+    is.numeric(order) || (
+      is.character(order) &&
+      (order %in% c(
+        "skewness","allClass","anyClass", "Outlying","Skewed","Clumpy",
+        "Sparse", "Striated", "Convex", "Skinny", "Stringy","Monotonic"
+      ))
+    )) ) {
+    stop(str_c(
+      "invalid value for 'order'; must either be a vector of column indices or one of ",
+      "'skewness','allClass','anyClass','Outlying','Skewed','Clumpy','Sparse','Striated',",
+      "'Convex','Skinny','Stringy','Monotonic'"
+    ))
   }
 
   if(!(is.logical(showPoints))) {
-    stop("invalid value for showPoints; must be a logical operator")
+    stop("invalid value for 'showPoints'; must be a logical operator")
   }
 
   alphaLinesIsCharacter <- is.character(alphaLines)
   if(alphaLinesIsCharacter) {
+    if(!(alphaLines %in% names(data))) {
+      stop("'alphaLines' column is missing in data")
+    }
+
     alphaRange <- range(data[,alphaLines])
     if (any(is.na(alphaRange))) {
-      if(alphaLines %in% names(data)) {
-        stop("missing data in alphaLines column")
-      } else {
-        stop("alphaLines column is missing in data")
-      }
+      stop("missing data in 'alphaLines' column")
     }
 
     if (alphaRange[1] < 0 || alphaRange[2] > 1) {
-      stop("invalid value for alphaLines column; max range must be from 0 to 1")
+      stop("invalid value for 'alphaLines' column; max range must be from 0 to 1")
     }
     alphaVar <- data[,alphaLines]
 
-  } else if((alphaLines < 0) || (alphaLines > 1)) {
-    stop("invalid value for alphaLines; must be a scalar value between 0 and 1")
+  } else if ((alphaLines < 0) || (alphaLines > 1)) { # nolint
+    stop("invalid value for 'alphaLines'; must be a scalar value between 0 and 1")
   }
 
-  if(!(is.logical(boxplot))) {
-    stop("invalid value for boxplot; must be a logical operator")
+  if (!(is.logical(boxplot))) {
+    stop("invalid value for 'boxplot'; must be a logical operator")
   }
 
+  if (is.logical(splineFactor)) {
+    if (splineFactor) {
+      splineFactor <- 3
+    } else {
+      splineFactor <- 0
+    }
+  } else if (! is.numeric(splineFactor)) {
+    stop("invalid value for 'splineFactor'; must be a logical or numeric value")
+  }
+
+
   ### Setup ###
   if(!is.null(groupColumn)) {
     if(is.numeric(groupColumn)) {
       groupCol <- names(data)[groupColumn]
-    } else
+    } else {
       groupCol <- groupColumn
+    }
     groupVar <- data[,groupCol]
   }
-  data <- data[,columns]
+
+  if (is.character(columns)) {
+    columns_ <- c()
+    for (colPos in seq_along(columns)) {
+      columns_[colPos] <- which(colnames(data) == columns[colPos])
+    }
+    columns <- columns_
+  }
+  # data <- data[, columns]
 
   # Change character vars to factors
-  vartypes <- get.VarTypes(data)
-  char.vars <- names(vartypes)[vartypes == "character"]
-  if(length(char.vars) >= 1) {
-    for(i in 1:length(char.vars)) {
-      data[,char.vars[i]] <- factor(data[,char.vars[i]])
+  char.vars <- column_is_character(data)
+  if (length(char.vars) >= 1) {
+    for (char.var in char.vars) {
+      data[,char.var] <- factor(data[,char.var])
     }
   }
 
   # Change factors to numeric
-  vartypes <- get.VarTypes(data)
-  fact.vars <- names(vartypes)[vartypes == "factor"]
-  if(length(fact.vars) >= 1) {
-    for(i in 1:length(fact.vars)) {
-      data[,fact.vars[i]] <- as.numeric(data[,fact.vars[i]])
+  fact.vars <- column_is_factor(data)
+  if (length(fact.vars) >= 1) {
+    for (fact.var in fact.vars) {
+      data[,fact.var] <- as.numeric(data[,fact.var])
     }
   }
 
@@ -237,142 +287,191 @@ ggparcoord <- function(
   # being plotted as numeric)
   saveData2 <- data
 
-  data$.ID <- as.factor(1:dim(data)[1])
-  data$anyMissing <- apply(data,1,function(x) { any(is.na(x)) })
-  p <- c(dim(data)[2]-1,dim(data)[2])
+  p <- c(ncol(data) + 1, ncol(data) + 2)
+  data$.ID <- as.factor(1:nrow(data))
+  data$anyMissing <- apply(is.na(data[,columns]), 1, any)
+  columnsPlusTwo <- c(columns, p)
+
+  inner_rescaler_default <- function (x, type = "sd", ...) {
+    # copied directly from reshape because of import difficulties :-(
+    # rescaler.default
+    switch(type,
+      rank = rank(x, ...),
+      var = ,
+      sd = (x - mean(x,na.rm = TRUE)) / sd(x, na.rm = TRUE),
+      robust = (x - median(x,na.rm = TRUE)) / mad(x, na.rm = TRUE),
+      I = x,
+      range = (x - min(x, na.rm = TRUE)) / diff(range(x, na.rm = TRUE))
+    )
+  }
+  inner_rescaler <- function(x, type = "sd", ...) {
+    # copied directly from reshape because of import difficulties :-(
+    # rescaler.data.frame
+    continuous <- sapply(x, is.numeric)
+    if (any(continuous)) {
+      if (type %in% c("sd", "robust", "range")) {
+        # indicating columns containing only one single value
+        singleVal <- sapply(x, function(col){
+          if (length(unique(col)) == 1) {
+            TRUE
+          } else {
+            FALSE
+          }
+        })
+        ind          <- continuous & !singleVal
+        x[ind]       <- lapply(x[ind], inner_rescaler_default, type = type, ...)
+        x[singleVal] <- 1
+      } else {
+        x[continuous] <- lapply(x[continuous], inner_rescaler_default, type = type, ...)
+      }
+    }
+    x
+  }
 
   ### Scaling ###
-  if(tolower(scale) == "std") {
-    data <- rescaler(data, type = "sd")
-  }
-  else if(tolower(scale) == "robust") {
-    data <- rescaler(data,type="robust")
-  }
-  else if(tolower(scale) == "uniminmax") {
-    data <- rescaler(data,type="range")
-  }
-  #else if(tolower(scale) == "globalminmax") {
-  #  data[,-p] <- data[,-p] - min(data[,-p])
-  #  data[,-p] <- data[,-p]/max(data[,-p])
-  #}
-  else if(tolower(scale) == "center") {
-    data <- rescaler(data,type="range")
-    data[,-p] <- apply(data[,-p],2,function(x){
-      x <- x - eval(parse(text=paste(scaleSummary,"(x,na.rm=TRUE)",sep="")))
-    })
+  if (tolower(scale) %in% c("std", "robust", "uniminmax", "center")) {
+    rescalerType <- c(
+      "std" = "sd",
+      "robust" = "robust",
+      "uniminmax" = "range",
+      "center" = "range"
+    )[tolower(scale)]
+    data[, columnsPlusTwo] <- inner_rescaler(data[, columnsPlusTwo], type = rescalerType)
+
+    if (tolower(scale) == "center") {
+      data[, columns] <- apply(data[, columns], 2, function(x){
+        x <- x - eval(parse(text=paste(scaleSummary,"(x,na.rm=TRUE)",sep="")))
+      })
+    }
+
   }
 
   ### Imputation ###
-  if(tolower(missing) == "exclude") {
-    dataCompleteCases <- complete.cases(data)
+  if (tolower(missing) == "exclude") {
+    dataCompleteCases <- complete.cases(data[, columnsPlusTwo])
 
-    if(!is.null(groupColumn)) {
+    if (!is.null(groupColumn)) {
       groupVar <- groupVar[dataCompleteCases]
     }
 
-    if(alphaLinesIsCharacter) {
+    if (alphaLinesIsCharacter) {
       alphaVar <- alphaVar[dataCompleteCases]
     }
 
-    data <- data[dataCompleteCases,]
+    data <- data[dataCompleteCases, ]
   }
-  else if(tolower(missing) == "mean") {
-     data[,-p] <- apply(data[,-p],2,function(x) {
-      if(any(is.na(x))) x[is.na(x)] <- mean(x,na.rm=TRUE)
-      return(x)
-    })
-  }
-  else if(tolower(missing) == "median") {
-     data[,-p] <- apply(data[,-p],2,function(x) {
-      if(any(is.na(x))) x[is.na(x)] <- median(x,na.rm=TRUE)
-      return(x)
-    })
-  }
-  else if(tolower(missing) == "min10") {
-     data[,-p] <- apply(data[,-p],2,function(x) {
-      if(any(is.na(x))) x[is.na(x)] <- .9*min(x,na.rm=TRUE)
-      return(x)
-    })
-  }
-  else if(tolower(missing) == "random") {
-     data[,-p] <- apply(data[,-p],2,function(x) {
-      if(any(is.na(x))) {
+  else if (tolower(missing) %in% c("mean", "median", "min10", "random")) {
+    missingFns <- list(
+      mean = function(x) {
+        mean(x, na.rm = TRUE)
+      },
+      median = function(x) {
+        median(x, na.rm = TRUE)
+      },
+      min10 = function(x){
+        0.9 * min(x, na.rm = TRUE)
+      },
+      random = function(x) {
         num <- sum(is.na(x))
         idx <- sample(which(!is.na(x)),num,replace=TRUE)
-        x[is.na(x)] <- x[idx]
+        x[idx]
+      }
+    )
+    missing_fn <- missingFns[[tolower(missing)]]
+    data[,columns] <- apply(data[,columns], 2, function(x) {
+      if (any(is.na(x))){
+        x[is.na(x)] <- missing_fn(x)
       }
       return(x)
     })
+
   }
 
   ### Scaling (round 2) ###
   # Centering by observation needs to be done after handling missing values
   #   in case the observation to be centered on has missing values
-  if(tolower(scale) == "centerobs") {
-    data <- rescaler(data,type="range")
-    data[,-p] <- apply(data[,-p],2,function(x){
+  if (tolower(scale) == "centerobs") {
+    data[, columnsPlusTwo] <- inner_rescaler(data[, columnsPlusTwo],type="range")
+    data[, columns] <- apply(data[,columns],2,function(x){
       x <- x - x[centerObsID]
     })
   }
 
-  meltIDVars <- c(".ID", "anyMissing")
-  if(!is.null(groupColumn)) {
-    data <- cbind(data,groupVar)
-    names(data)[dim(data)[2]] <- groupCol
+  # meltIDVars <- c(".ID", "anyMissing")
+  meltIDVars <- colnames(data)[-columns]
+
+  if (!is.null(groupColumn)) {
+    # data <- cbind(data,groupVar)
+    # names(data)[dim(data)[2]] <- groupCol
 
     meltIDVars <- c(groupCol, meltIDVars)
   }
 
-  if(alphaLinesIsCharacter) {
+  if (alphaLinesIsCharacter) {
     data <- cbind(data, alphaVar)
     names(data)[dim(data)[2]] <- alphaLines
     meltIDVars <- c(meltIDVars, alphaLines)
   }
 
-  data.m <- melt(data,id.vars=meltIDVars)
+  # if(is.list(mapping)) {
+  #   mappingNames <- names(mapping)
+  # }
 
+  data.m <- melt(data, id.vars = meltIDVars, measure.vars = columns)
 
   ### Ordering ###
-  if(length(order) > 1) {
-     if(is.numeric(order)) {
-       data.m$variable <- factor(data.m$variable,levels=names(saveData)[order])
-     } else {
-       data.m$variable <- factor(data.m$variable,levels=order)
-     }
+  if (length(order) > 1 & is.numeric(order)) {
+     data.m$variable <- factor(data.m$variable,levels=names(saveData)[order])
   }
-  else if(order %in% c("Outlying","Skewed","Clumpy","Sparse","Striated","Convex","Skinny",
+  else if (order %in% c("Outlying","Skewed","Clumpy","Sparse","Striated","Convex","Skinny",
     "Stringy","Monotonic")) {
-    require(scagnostics)
-    scag <- scagnostics(saveData2)
-    data.m$variable <- factor(data.m$variable,levels=scagOrder(scag,names(saveData2),order))
+
+    require_pkgs("scagnostics")
+    scag <- scagnostics::scagnostics(saveData2)
+    data.m$variable <- factor(data.m$variable, levels = scag_order(scag, names(saveData2), order))
   }
-  else if(tolower(order) == "skewness") {
+  else if (tolower(order) == "skewness") {
     abs.skew <- abs(apply(saveData2,2,skewness))
-    data.m$variable <- factor(data.m$variable,levels=names(abs.skew)[order(abs.skew,decreasing=TRUE)])
+    data.m$variable <- factor(
+      data.m$variable,
+      levels = names(abs.skew)[order(abs.skew,decreasing=TRUE)]
+    )
   }
-  else if(tolower(order) == "allclass") {
+  else if (tolower(order) == "allclass") {
     f.stats <- rep(NA,length(columns))
-    names(f.stats) <- names(saveData2)
+    names(f.stats) <- names(saveData2[columns])
     for(i in 1:length(columns)) {
       f.stats[i] <- summary(lm(saveData2[,i] ~ groupVar))$fstatistic[1]
     }
-    data.m$variable <- factor(data.m$variable,levels=names(f.stats)[order(f.stats,decreasing=TRUE)])
+    data.m$variable <- factor(
+      data.m$variable,
+      levels = names(f.stats)[order(f.stats,decreasing=TRUE)]
+    )
   }
-  else if(tolower(order) == "anyclass") {
+  else if (tolower(order) == "anyclass") {
     axis.order <- singleClassOrder(groupVar,saveData2)
     data.m$variable <- factor(data.m$variable,levels=axis.order)
   }
 
-  if(!is.null(groupColumn)) {
-    mapcall <- paste("aes_string(x='variable',y='value',group='.ID',colour='",groupCol,"')",sep="")
+  if (!is.null(groupColumn)) {
+    mapping2 <- aes_string(
+      x = "variable",
+      y = "value",
+      group = ".ID",
+      colour = as.character(substitute(groupCol))
+    )
   } else {
-    mapcall <- paste("aes_string(x='variable',y='value',group='.ID')",sep="")
+    mapping2 <- aes_string(
+      x = "variable",
+      y = "value",
+      group = ".ID"
+    )
   }
-  mapping2 <- eval(parse(text=mapcall))
-  mapping2 <- addAndOverwriteAes(mapping2,mapping)
+  mapping2 <- add_and_overwrite_aes(mapping2,mapping)
+  # mapping2 <- add_and_overwrite_aes(aes_string(size = I(0.5)), mapping2)
   p <- ggplot(data=data.m,mapping=mapping2)
 
-  if(!is.null(shadeBox)) {
+  if (!is.null(shadeBox)) {
     # Fix so that if missing = "min10", the box only goes down to the true min
     d.sum <- ddply(data.m,.(variable),summarize,
       min = min(value),
@@ -380,25 +479,74 @@ ggparcoord <- function(
     p <- p + geom_linerange(data=d.sum,size=I(10),col=shadeBox,
       mapping=aes(y=NULL,ymin=min,ymax=max,group=variable))
   }
+
   if (boxplot)
     p <- p + geom_boxplot(mapping=aes(group=variable),alpha=0.8)
 
-  if (showPoints)
-    p <- p + geom_point()
-
-  if(!is.null(mapping2$size)) {
+  if (!is.null(mapping2$size)) {
     lineSize <- mapping2$size
   } else {
     lineSize <- 0.5
   }
 
-  if (alphaLinesIsCharacter) {
-    p <- p +
-      geom_line(aes_string(alpha = alphaLines), size = lineSize) +
-      scale_alpha(range = alphaRange)
+  if (splineFactor > 0) {
+    data.m$ggally_splineFactor <- splineFactor
+    if (class(splineFactor) == "AsIs") {
+      data.m <- ddply(
+        data.m, ".ID", transform,
+        spline = spline(variable, value, n = ggally_splineFactor[1])
+      )
+    } else {
+      data.m <- ddply(
+        data.m, ".ID", transform,
+        spline = spline(variable, value, n = length(variable) * ggally_splineFactor[1])
+      )
+    }
+
+    linexvar <- "spline.x"
+    lineyvar <- "spline.y"
+
+    if (alphaLinesIsCharacter) {
+      p <- p +
+        geom_line(
+          aes_string(x = linexvar, y = lineyvar, alpha = alphaLines),
+          size = lineSize,
+          data = data.m
+        ) +
+        scale_alpha(range = alphaRange)
+
+    } else {
+      p <- p +
+        geom_line(
+          aes_string(x = linexvar, y = lineyvar),
+          alpha = alphaLines,
+          size = lineSize,
+          data = data.m
+        )
+    }
+
+    if (showPoints) {
+      p <- p + geom_point(aes(x = as.numeric(variable), y = value))
+    }
+
+    xAxisLabels <- levels(data.m$variable)
+    # while continuous data, this makes it present like it's discrete
+    p <- p + scale_x_discrete(breaks = seq_along(xAxisLabels), labels = xAxisLabels)
 
   } else {
-    p <- p + geom_line(alpha = alphaLines, size = lineSize)
+    if (alphaLinesIsCharacter) {
+      p <- p +
+        geom_line(aes_string(alpha = alphaLines), size = lineSize, data = data.m) +
+        scale_alpha(range = alphaRange)
+
+    } else {
+      # p <- p + geom_line(alpha = alphaLines, size = lineSize)
+      p <- p + geom_line(alpha = alphaLines)
+    }
+
+    if (showPoints) {
+      p <- p + geom_point()
+    }
   }
 
   if (title != "") {
@@ -410,12 +558,19 @@ ggparcoord <- function(
 
 #' Get vector of variable types from data frame
 #'
+#' @keywords internal
 #' @param df data frame to extract variable types from
 #' @author Jason Crowley \email{crowley.jason.s@@gmail.com}
 #' @return character vector with variable types, with names corresponding to
 #'   the variable names from df
-get.VarTypes <- function(df) {
-  return(unlist(lapply(unclass(df),class)))
+column_is_character <- function(df) {
+  x <- unlist(lapply(unclass(df),is.character))
+  names(x)[x]
+}
+#' @rdname column_is_character
+column_is_factor <- function(df) {
+  x <- unlist(lapply(unclass(df),is.factor))
+  names(x)[x]
 }
 
 #' Find order of variables
@@ -426,31 +581,68 @@ get.VarTypes <- function(df) {
 #' @param scag \code{scagnostics} object
 #' @param vars character vector of the variables to be ordered
 #' @param measure scagnostics measure to order according to
-#' @author Jason Crowley \email{crowley.jason.s@@gmail.com}
+#' @author Barret Schloerke
 #' @return character vector of variable ordered according to the given
 #'   scagnostic measure
-scagOrder <- function(scag, vars, measure) {
-  p <- length(vars)
-  scag <- sort(scag[measure,],decreasing=TRUE)
-  d.scag <- data.frame(var1=NA,var2=NA,val=scag)
-  for (i in 1:dim(d.scag)[1]) {
-    d.scag$var1[i] <- substr(names(scag)[i],1,regexpr(" * ",names(scag)[i])-1)
-    d.scag$var2[i] <- substr(names(scag)[i],regexpr(" * ",names(scag)[i])+3,nchar(names(scag)[i]))
+scag_order <- function(scag, vars, measure) {
+
+  scag <- sort(scag[measure, ], decreasing = TRUE)
+  scagNames <- names(scag)
+
+  # retrieve all names.  assume name doesn't contain a space
+  nameLocs <- regexec("^([^ ]+) \\* ([^ ]+)$", scagNames)
+
+  colNames <- lapply(seq_along(nameLocs), function(i) {
+    nameLoc <- nameLocs[[i]]
+    scagName <- scagNames[[i]]
+    # retrieve the column name from "FIRSTNAME * SECONDNAME"
+    substr(rep(scagName, 2), nameLoc[-1], nameLoc[-1] + attr(nameLoc, "match.length")[-1] - 1)
+  })
+
+  ret <- c()
+  colNamesLength <- length(colNames)
+  colNameValues <- unlist(colNames)
+  for (i in seq_along(colNames)) {
+    cols <- colNames[[i]]
+    colsUsed <- cols %in% ret
+    # if none of the columns have been added...
+    if (colsUsed[1] == FALSE && colsUsed[2] == FALSE) {
+      # find out which column comes next in the set, append that one first
+      if (i < colNamesLength) {
+        remainingColumns <- colNameValues[(2 * (i + 1)):(2 * colNamesLength)]
+        col1Pos <- which.min(cols[1] == remainingColumns)
+        col2Pos <- which.min(cols[2] == remainingColumns)
+        if (col2Pos < col1Pos) {
+          cols <- rev(cols)
+        }
+        ret <- append(ret, cols)
+      } else {
+        # nothing left in set, append both
+        ret <- append(ret, cols)
+      }
+
+    # if only the first hasn't been added...
+    } else if (colsUsed[1] == FALSE) {
+      ret <- append(ret, cols[1])
+
+    # if only the second hasn't been added...
+    } else if (colsUsed[2] == FALSE) {
+      ret <- append(ret, cols[2])
+    }
   }
-  a <- c(d.scag$var1[1],d.scag$var2[1])
-  d.scag <- d.scag[-1,]
-  a <- a[order(c(min(grep(a[1],rownames(d.scag))),min(grep(a[2],rownames(d.scag)))),
-    decreasing=TRUE)]
-  d.scag <- d.scag[-grep(a[1],rownames(d.scag)),]
-  while(length(a) < (p-1)) {
-    k <- length(a)
-    a[k+1] <- d.scag[1,1:2][!(a[k] == d.scag[1,1:2])]
-    d.scag <- d.scag[-grep(a[k],rownames(d.scag)),]
+
+  if (length(ret) != length(vars)) {
+    stop(str_c(
+      "Could not compute a correct ordering: ",
+      length(vars) - length(ret), " values are missing. ",
+      "Missing: ", paste0(vars[! (vars %in% ret)], collapse = ", ")
+    ))
   }
-  a[p] <- vars[!(vars %in% a)]
-  return(a)
+
+  return(ret)
 }
 
+
 #' Order axis variables
 #'
 #' Order axis variables by separation between one class and the rest
@@ -462,6 +654,7 @@ scagOrder <- function(scag, vars, measure) {
 #'   of looking for separation between any class and the rest, will only look for
 #'   separation between this class and the rest
 #' @author Jason Crowley \email{crowley.jason.s@@gmail.com}
+#' @importFrom stats lm
 #' @return character vector of names of axisVars ordered such that the first
 #'   variable has the most separation between one of the classes and the rest, and
 #'   the last variable has the least (as measured by F-statistics from an ANOVA)
@@ -496,6 +689,6 @@ skewness <- function(x) {
   x <- x[!is.na(x)]
   xbar <- mean(x)
   n <- length(x)
-  skewness <- (1/n)*sum((x-xbar)^3)/((1/n)*sum((x-xbar)^2))^(3/2)
+  skewness <- (1 / n) * sum( (x - xbar) ^ 3) / ( (1 / n) * sum( (x - xbar) ^ 2)) ^ (3 / 2)
   return(skewness)
 }
